classdef CEM_forward_model < Forward_model
    properties
        electrode_system;
    end
    methods (Hidden = true)
        function postprocess_system_assembly(obj) % Overrides
            electrode_triplets = obj.electrode_model.system_appendix(obj.mesh);
            ss = obj.system_size();
            
            obj.electrode_system = sparse(electrode_triplets.rows, ...
                                          electrode_triplets.cols, ...
                                          electrode_triplets.vals, ...
                                          ss(1), ss(2));

            obj.system = obj.conductivity_system + obj.electrode_system;
        end

        function postprocess_solution(obj) % Overrides
            obj.domain_derivative_potentials = obj.solution(1:obj.solution_structure.domain(1), :);
        end
    end

    methods
        function obj = CEM_forward_model(mesh, domain_parametrization, electrodes, current_plan)
            obj = obj@Forward_model(mesh, domain_parametrization, electrodes, current_plan);
            obj.solution_structure.electrode_potentials = [electrodes.Ne - 1, electrodes.Ne - 1];
            obj.perturbation_structure.electrode_conductances = [electrodes.Nperturb];
        end

        function perturb(obj, eta)
            perturb@Forward_model(obj, eta(1:obj.perturbation_structure.domain));
            obj.electrode_model.perturb(eta(end - obj.perturbation_structure.electrode_conductances + 1:end));
        end

        function solve_derivatives(obj)
            solve_derivatives@Forward_model(obj);

            el_ders = obj.electrode_model.electrode_derivatives(obj.solution, obj.mesh);
            obj.gradient(:, :, obj.perturbation_structure.domain+1 : end+obj.perturbation_structure.electrode_conductances) = el_ders;
        end
    end
end
