classdef (Abstract) Domain_parametrization_interface < matlab.mixin.Copyable
    properties
        conductivity_bounds;
    end
    methods(Abstract)
        % obj = parametrization object
        % gx = global x
        % gy = global y
        % tx = reference triangle x
        % ty = reference triangle y
        % tid = point's triangle index
        % mesh = mesh structure. Should contained e.g. t2p matrix.
        % RETURNS either size(Nt, 1), or size(Nt, size(tx, 2)) array of conductances.
        conds = conductances(obj, gx, gy, tx, ty, tid, mesh)

        % Set parametrization so that resulting conductance field is
        % constant.
        set_constant(obj, val)

        % p = externally visible parameters. Must be consistent with
        % methods below.
        p = get_params(obj)
        
        % Number of externally visible parameters. Must be consistent with
        % params, perturb(..) and derivative(..).
        N = Nperturb(obj)

        % Perturb parametrization by parameter vector eta. Eta is in the
        % same units as derivatives generated by derivative(..) function.
        perturb(obj, eta)

        % el_D = Elementwise derivative as N x N x Nt tensor
        derivatives = derivative(obj, el_D, mesh)
    end
    
    methods
        function p = get_params_as_conductivity(obj)
            p = obj.get_params();
        end
    end
end
