classdef (Abstract) Functional_electrode_parametrization_interface < matlab.mixin.Copyable
    methods(Abstract)
        % obj = electrode object
        % gx = global x
        % gy = global y
        % electrode_ind = electrode index
        % electrode_l = electrode arc length parametrization (normalized)
        % bedge_l = single edge length parametrization = quadrature points = Nqp x 1
        % boundary_l = full boundary arc length parametrization (absolute)
        % RETURNS size(gx) array of conductances.
        conds = conductances(obj, gx, gy, electrode_ind, electrode_l, bedge_l, boundary_l)
        
        % Set parametrization so that each electrode has net contact
        % resistance of Rcs. Rcs may be a single value or a vector of N
        % elements where N is the number of electrodes
        set_resistors(obj, Rcs)
        
        % p = externally visible parameters. Must be consistent with
        % methods below.
        p = get_params(obj)
        
        % Number of externally visible parameters. Must be consistent with
        % params, perturb(..) and derivative(..).
        N = Nperturb(obj)

        % Perturb parametrization by parameter vector eta. Eta is in the
        % same units as derivatives generated by derivative(..) function.
        perturb(obj, eta)

        % Compute derivative for fixed pair (U-u)(V-v). I.e. compute the
        % integral \partial_t U(\sigma, \zeta(t), I; \eta) \cdot \tilde{I} = - \int_{\partial \Omega} \frac{\partial \zeta(t)}{\partial_t} \, (U - u) (\tilde{U} - \tilde{u}) \, {\rm d} S.
        % Xb = quadrature points (per mesh edge)
        % Wb = quadrature weights (per mesh edge)
        % vdrop2 = pointwise values of (U-u)(V-v) for the quadrature for fixed pairs (U,u) and (V,v). Nedges x length(Xb)
        % mesh = mesh
        derivatives = derivative(obj, Xb, Wb, vdrop2, mesh)
    end
end
