classdef Nodewise_log_domain_parametrization < Nodewise_conductivity_domain_parametrization
    methods
        function obj = Nodewise_log_domain_parametrization(mesh)
            obj = obj@Nodewise_conductivity_domain_parametrization(mesh);
        end

        % Set parametrization so that resulting conductance field is
        % constant.
        function set_constant(obj, val)
            obj.nodewise_conductivity(:) = val;
        end

        % p = externally visible parameters. Must be consistent with
        % methods below.
        function p = get_params(obj)
            p = log(obj.nodewise_conductivity);
        end
        
        function p = get_params_as_conductivity(obj)
            p = obj.nodewise_conductivity;
        end

        % Perturb parametrization by parameter vector eta. Eta is in the
        % same units as derivatives generated by derivative(..) function.
        function perturb(obj, eta)
            obj.nodewise_conductivity(:) = min(obj.conductivity_bounds(2), ...
                                               max(obj.conductivity_bounds(1), ...
                                               obj.nodewise_conductivity(:) .* exp(eta(:))));
        end


        % el_D = Elementwise derivative as N x N x Nt tensor
        function derivatives = derivative(obj, el_D, mesh)
            % Nodewise parametrization
            % Vval is a vector of 1/2 = value at sample points.
            % The "* (W/3)" is same as ".* (W')", as the sample
            % values are always the same. W/3 just provides only
            % one column which must be added twice separately for
            % each node in a triangle.
            % Could as well just take etaUfhatUghatEl ./ 6;
            %etaUfhatUghatV = (Vval.*(dLix.*dLjx + dLiy.*dLjy)) * (W/3).*abs(obj.mesh.affine.detA);
            %etaUfhatUghatV = reshape(etaUfhatUghatV, 1, 1, length(etaUfhatUghatV));
            derivatives = accumndarray(el_D, mesh.p2t', 1./3 * ones(size(mesh.p2t')));
            expkappa = reshape(obj.nodewise_conductivity(:), 1, 1, []);
            derivatives(:, :, :) = derivatives(:, :, :) .* expkappa;
        end
    end
end
