classdef P1CEM_square_conductivity_electrode_parametrization < Functional_electrode_parametrization_interface
    properties
        P1_elmask;
        boundary_conductances; % Nbp x 1, Must be 0 at end of electrodes.
        boundary_conductance_limits = [0.00000, 100000];
        mesh;
    end
    properties (Hidden = true)
        Nparams;
    end

    methods
        function obj = P1CEM_square_conductivity_electrode_parametrization(mesh)
            % Remove first and last nodes from bedgestream to force
            % conductance = 0 at end of electrode.
            obj.P1_elmask = min(circshift(mesh.bedgestream(:, 3), [-1 0]), mesh.bedgestream(:, 3));
            obj.P1_elmask = min(circshift(mesh.bedgestream(:, 3), [1 0]), obj.P1_elmask);
            obj.boundary_conductances = double(logical(obj.P1_elmask)) * 1;
            obj.Nparams = nnz(obj.P1_elmask);
            obj.mesh = mesh;
        end

        function conds = conductances(obj, gx, gy, electrode_ind, electrode_l, bedge_l, boundary_l);
            % Assumes obj.boundary_conductances are exactly 0 at rows
            % corresponding to ends of electrodes. This must be enforced in
            % obj.perturb().
            conds = obj.boundary_conductances * (1 - bedge_l) + circshift(obj.boundary_conductances, [-1 0]) * bedge_l;
        end

        function set_resistors(obj, Rcs)
            if length(Rcs) == 1
                Rcs = Rcs * ones(max(obj.mesh.bedgestream(:, 3)), 1);
            end
            
            [vf, idf] = unique(obj.mesh.bedgestream(:, 3), 'first');
            [vl, idl] = unique(obj.mesh.bedgestream(:, 3), 'last');
            [~, ordf] = sort(vf);
            [~, ordl] = sort(vl);
            idf = idf(ordf); idf = idf(2:end);
            idl = idl(ordl); idl = idl(2:end);
            ellens = obj.mesh.bedgestream(idl, 4) - obj.mesh.bedgestream(idf, 4);

            bconductances = [0; 1./(Rcs(:).*ellens)];
            obj.boundary_conductances = bconductances(obj.mesh.bedgestream(:, 3) + 1);
            obj.boundary_conductances = obj.boundary_conductances .* logical(obj.P1_elmask);
%             obj.boundary_conductance_limits = [obj.boundary_conductance_limits(1), 5 * max(bconductances)];
        end
        
        function p = get_params(obj)
            p = sqrt(obj.boundary_conductances(logical(obj.P1_elmask)));
        end
        
        function N = Nperturb(obj)
            N = obj.Nparams;
        end

        function perturb(obj, eta)
            obj.boundary_conductances(logical(obj.P1_elmask)) = min(max((sqrt(obj.boundary_conductances(logical(obj.P1_elmask))) + eta).^2, ...
                                                                        obj.boundary_conductance_limits(1)), ...
                                                                    obj.boundary_conductance_limits(2));
        end

        % Compute derivative for fixed pair (U-u)(V-v). I.e. compute the
        % integral \partial_t U(\sigma, \zeta(t), I; \eta) \cdot \tilde{I} = - \int_{\partial \Omega} \frac{\partial \zeta(t)}{\partial_t} \, (U - u) (\tilde{U} - \tilde{u}) \, {\rm d} S.
        % Xb = quadrature points (per mesh edge)
        % Wb = quadrature weights (per mesh edge)
        % vdrop2 = pointwise values of (U-u)(V-v) for the quadrature for fixed pairs (U,u) and (V,v). Nedges x length(Xb)
        % mesh = mesh
        function derivatives = derivative(obj, Xb, Wb, vdrop2, mesh)
            derivatives = zeros(obj.Nparams, 1);
            
            vals = -((fliplr(Xb) .* vdrop2) * Wb .* mesh.affine.detAb + circshift((Xb .* vdrop2) * Wb .* mesh.affine.detAb, [1, 0]));
            derivatives = vals(logical(obj.P1_elmask)) .* sqrt(obj.boundary_conductances(logical(obj.P1_elmask))) .* 2;
        end
    end
end