function gauss = gauss_P1CEM_dist_prior(mesh, P1CEM_parametrization, corrdist, weight)
    % GAUSS_P1CEM_DIST_PRIOR Electrode arc-length distance based covariance.
    %   Ignores electrode arc end points.
    %   out = GAUSS_P1CEM_DIST_PRIOR(mesh, electrodes, correlation_distance, weight)

    elmask = mesh.bedgestream(:, 3);
    elmask_interior = P1CEM_parametrization.P1_elmask;
    gauss = zeros(size(logical(elmask), 1), size(logical(elmask), 1));

    for elind = 1:max(elmask)
        elindmask = elmask == elind;
        elindmask_interior = elmask_interior == elind;
        elbrdmask = elindmask_interior ~= elindmask;
        arclength = mesh.bedgestream(elindmask, 4);
        distsquares = (arclength - arclength').^2;
        gauss(elindmask, elindmask) = weight^2 * exp(-0.5 * (distsquares)./(corrdist.^2));

        gamma11 = gauss(elindmask_interior, elindmask_interior);
        gamma12 = gauss(elindmask_interior, elbrdmask);
        gamma21 = gauss(elbrdmask, elindmask_interior);
        gamma22 = gauss(elbrdmask, elbrdmask);
        
        gauss(elindmask_interior, elindmask_interior) = gamma11 - gamma12 * inv(gamma22) * gamma21;
        %gauss(elindmask, elindmask) = mesh.bedgestream(elindmask, 2)' .* gauss(elindmask, elindmask) .* mesh.bedgestream(elindmassk, 2);
        
    end
    gauss = gauss(logical(elmask_interior), logical(elmask_interior));
end
