finalerrs = [];

CEM_gauss_mids = maybeor('CEM_gauss_mids', []);
CEM_gauss_finalerrs = maybeor('CEM_gauss_finalerrs', []);


for prior_dls = maybeor('prior_dls', 0.030);
for prior_dws = maybeor('prior_dws', 100);

el_parametrization = Resistance_CEM_electrode_parametrization(mesh);
el_widths = coly(maybeor('el_width ./ (el_extension_width + el_width)', 1.0));
el_parametrization.electrode_domain = maybeor('[clamp(0.5 - (ones(Ne, 1) .* el_widths ./ 2), 0, 1 - el_widths), clamp(0.5 + (ones(Ne, 1) .* el_widths ./ 2), el_widths, 1)]', [zeros(Ne, 1), ones(Ne, 1)]);
el_parametrization.set_resistors(1000);

electrodes = Functional_electrodes(mesh, el_parametrization);

dom_parametrization = Nodewise_log_domain_parametrization(mesh);
dom_parametrization.conductivity_bounds = [10^-6 10^6];
dom_parametrization.set_constant(10^-3);

fm = CEM_forward_model(mesh, dom_parametrization, electrodes, current_plan);

clear FINISH_FLAG;
% DISABLE_DRAW = 1;


fm.use_log = 1;
for snip = 1 % Figures
    if maybeor('not(DISABLE_DRAW)', 0)
        %h_cond = figure(1); colormap(mycolormap(100)); clf;
        h_iter = figure(2); colormap('gray'); clf;
        %h_grad = figure(3); colormap('gray'); clf;
        h_step = figure(4); colormap(mycolormap(255)); clf;
%         h_exact = figure(5); colormap('gray'); clf;
        h_electrodes = figure(6); clf;
        %h_volts = figure(7); clf;
    end
end

x0 = [fm.domain_parametrization.get_params();
      zeros(fm.electrode_model.Ne, 1)];

prior_dl = prior_dls;
prior_dw = prior_dws;
domain_prior = gauss_node_dist_prior(mesh, prior_dl, prior_dw);
domain_prior_inv = inv(domain_prior + eigs(domain_prior, 1) / 10^5 * eye(dom_parametrization.Nperturb()));

Ne = fm.electrode_model.Ne;
electrode_prior = eye(Ne);
L_electrode = chol(electrode_prior);

tikhonov = [domain_prior_inv, zeros(fm.perturbation_structure.domain, fm.perturbation_structure.electrode_conductances)
            zeros(fm.perturbation_structure.electrode_conductances, fm.perturbation_structure.domain), electrode_prior];

L = chol(tikhonov);

alpha_domain = 1;
alpha_electrode = 0;
a_vec = [alpha_domain * ones(fm.perturbation_structure.domain, 1);
         alpha_electrode * ones(fm.perturbation_structure.electrode_conductances, 1)];
aLL = sqrt(a_vec .* a_vec') .* (L' * L);


EIT_minimization();

finalerrs(end+1, :) = [mean(fm.domain_parametrization.get_params_as_conductivity()) / 0.05, ...
                       norm(accuratevolts(:) - volts(:)), ...
                       (fm.domain_parametrization.get_params() - x0(1:fm.perturbation_structure.domain))' * aLL(1:fm.perturbation_structure.domain, 1:fm.perturbation_structure.domain) * (fm.domain_parametrization.get_params() - x0(1:fm.perturbation_structure.domain)), ...
                       (fm.electrode_model.parametrization.get_params() - x0(fm.perturbation_structure.domain+1:end))' * aLL(fm.perturbation_structure.domain+1:end, fm.perturbation_structure.domain+1:end) * (fm.electrode_model.parametrization.get_params() - x0(fm.perturbation_structure.domain+1:end)), ...
                       prior_dl, prior_dw]

                   
[~, temp_mids] = fm.electrode_model.conductance_integrals();
CEM_gauss_mids(:, end+1) = temp_mids;
CEM_gauss_finalerrs(end+1, :) = finalerrs(end, :);

end % prior_dws
end % prior_dls
