% Expects existence of:
%  fm - the forward model
%  x0 - the initial guess
%  aLL - the covariance/prior matrix
%

clear BT_FINISH_FLAG

aLL_el = aLL(fm.perturbation_structure.domain+1:end, fm.perturbation_structure.domain+1:end);
aLL_dom = aLL(1:fm.perturbation_structure.domain, 1:fm.perturbation_structure.domain);

for iter = 1:50
    tic
    volts = evaluate(fm);
    fm.solve_derivatives();
    grad = fm.get_gradient();
    measgrad = zeros(size(grad, 1) * (size(grad, 2)), size(grad, 3));
    for cell = 1:size(grad, 3)
        measgrad(:, cell) = reshape((grad(:, :, cell)), [], 1);
    end
    M = measgrad' * measgrad;

    xj = [fm.domain_parametrization.get_params(); 
          fm.electrode_model.parametrization.get_params()];

    h = (M + aLL) \ (measgrad' * (accuratevolts(:) - volts(:)) + aLL * (x0 - xj));

    % Backtracking line search 
    bt_alpha = 2^0;
    bt_tau = 0.5;
    curr_dom_discrepancy = fm.domain_parametrization.get_params() - x0(1:fm.perturbation_structure.domain);
    curr_el_discrepancy = fm.electrode_model.parametrization.get_params() - x0(fm.perturbation_structure.domain+1:end);

    curr_residual = norm(accuratevolts(:) - volts(:))^2;
    curr_dom_cost = curr_dom_discrepancy' * aLL(1:fm.perturbation_structure.domain, 1:fm.perturbation_structure.domain) * curr_dom_discrepancy;
    curr_el_cost = curr_el_discrepancy' * aLL(fm.perturbation_structure.domain+1:end, fm.perturbation_structure.domain+1:end) * curr_el_discrepancy;

    curr_cost = curr_residual + curr_el_cost + curr_dom_cost;
    bt_best_cost = curr_cost;
    bt_best_alpha = 0;
    bt_improvement = false;

    while (true)
        bt_fm = copy(fm);
        bt_fm.perturb(bt_alpha * h);

        bt_volts = evaluate(bt_fm);
        bt_dom_discrepancy = bt_fm.domain_parametrization.get_params() - x0(1:fm.perturbation_structure.domain);
        bt_e_discrepancy = [bt_fm.electrode_model.parametrization.get_params()] - x0(fm.perturbation_structure.domain+1:end);

        bt_residual = norm(accuratevolts(:) - bt_volts(:))^2;
        bt_el_cost = bt_e_discrepancy' * aLL_el * bt_e_discrepancy;
        bt_dom_cost = bt_dom_discrepancy' * aLL_dom * bt_dom_discrepancy;
        
        bt_cost = bt_residual + bt_el_cost + bt_dom_cost;
        printverbose('alpha = %f, residual = %f, el_cost = %f, dom_cost = %f, cost = %f\n', bt_alpha, bt_residual, bt_el_cost, bt_dom_cost, bt_cost);
        if bt_cost < curr_cost
            bt_improvement = true;
        end
        if (bt_improvement & bt_cost > bt_best_cost)
            break;
        end
        if bt_cost < bt_best_cost
            bt_best_cost = bt_cost;
            bt_best_alpha = bt_alpha;
        end
        if (norm(bt_alpha * h) < 10^-4)
            BT_FINISH_FLAG = true;
            break;
        end
        bt_alpha = bt_tau * bt_alpha;
    end
%     fprintf("iter = %d, alpha = %f, norm(h * alpha) = %f\n", iter, bt_best_alpha, norm(bt_best_alpha * h));
    fm.perturb(bt_best_alpha * h);

    volts = evaluate(fm);
    printverbose("Iter net resistance \n")
    printverbose("%2.0f \t %5.3f \t %5.5f\n", [[1:max(mesh.bedgestream(:, 3))]' 1./electrodes.conductance_integrals() electrodes.conductance_integrals()]')
    printdebug("Mean domain conductivity = %f\n", mean(fm.domain_parametrization.get_params_as_conductivity()));
    printdebug("iter = %d,  step length = %f,  ||accuratevolts(:) - volts(:)|| = %f\n", iter, norm(bt_best_alpha*h), norm(accuratevolts(:) - volts(:)));
    printdebug("Iteration time: %f seconds\n", toc)
    if exist("BT_FINISH_FLAG", "var")
        break
    end % Exit iteration if no more improvement
end
clear BT_FINISH_FLAG bt_fm bt_volts bt_dom_discrepancy bt_e_discrepancy bt_residual bt_el_cost bt_dom_cost bt_cost bt_alpha bt_tau M aLL_dom aLL_el
printinfo("Final iterations = %d,  residual = %5.7f,  dom_cost = %5.7f,  el_cost = %5.7f.\n", iter, sqrt(curr_residual), sqrt(curr_dom_cost), sqrt(curr_el_cost))

function volts = evaluate(fme)
    fme.assemble_system();
    fme.assemble_loads();
    fme.solve_potentials();
    volts = fme.get_NtoD();
end
