% Verify:
% - All scripts use the same conductivity model (log conductivity)
% - Prior parameters are consistent across all experiments
% - Initial guesses are consistent across all experiments and reasonable.

VERBOSE = 1;
DISABLE_DRAW = 1;
midpoint_errors_hard = [];
log_contact_conductances_mean_hard = [];
log_contact_conductances_sd_hard = [];

CEM_gauss_mids = [];
CEM_gauss_finalerrs = [];
P1CEM_gauss_mids = [];
P1CEM_gauss_finalerrs = [];
Peak_gauss_mids = [];
Peak_gauss_finalerrs = [];
    
for el_extension_mm = [0 10 20]
    el_width = 0.02; 
    prior_dls = 0.03;
    prior_dws = 100;

    seeds.el_extension = 1;
    el_extension_width = el_extension_mm / 1000;
    el_pos_skews = maybeor('seededrand(seeds.el_extension, 16, 1)', rand(16, 1)) + 0.0*ones(16, 1);

    if el_extension_width == 0.0
        [mesh] = get_torso_mesh();
    else
        [mesh] = get_torso_mesh(el_extension_width, el_pos_skews);
    end
    el_extension_width = max(0.0, get_el_lens(mesh) - el_width);
    Ne = max(mesh.bedgestream(:, 3));

    d_case = '5';
    [current_plan, accuratevolts] = get_torso_datas(d_case);

    drawi = 10;
    fprintf("CEM\n");
    CEM_gauss_test();
    CEM_final_fm = fm.copy();

    fprintf("P1CEM\n");
    P1CEM_gauss_test();
    P1CEM_final_fm = fm.copy();

    fprintf("Peak\n");
    Peak_gauss_test();
    Peak_final_fm = fm.copy();


    fms = [CEM_final_fm, P1CEM_final_fm, Peak_final_fm];
    %%
    line_formats = {'#l2--', '#l1-', '#l6-.'};
    point_formats = {'#l2-', '#l1-', '#l6-'};


    myfigure(100 + el_extension_mm); clf;
    subplot(1, 3, 1); cla; title('CEM');
    draw_cond(CEM_final_fm);
    subplot(1, 3, 2); cla; title('P1CEM');
    draw_cond(P1CEM_final_fm);
    subplot(1, 3, 3); cla; title('Peak');
    draw_cond(Peak_final_fm);


    myfigure(41); clf;
    for fm = fms
        draw_boundary_volts(fm, drawi, line_formats{find(fms == fm)}, 'LineWidth', 1.5);
    end
    mylegend(line_formats(1:3), {'CEM', 'PL', 'PH'}, 'LineWidth', 1.5);

    myfigure(46); clf;
    ints = [];
    mids = [];
    for fm = fms
        [ints(:, end+1), mids(:, end+1)] = fm.electrode_model.conductance_integrals();
    end
    exact_ends = (1-el_pos_skews(:)) .* (1 - el_width./(el_extension_width(:)+el_width)) + el_width ./ (el_extension_width(:)+el_width);
    exact_starts = (1-el_pos_skews(:)) .* (1 - el_width./(el_extension_width(:)+el_width));
    exact_centers = (exact_ends + exact_starts) / 2;
    for pl = 1:Ne
        ph = subplot(8+1, ceil(Ne/8), pl); hold on;
        for fm_ind = 1:length(fms(1:3))
            fm = fms(fm_ind);
            draw_contact(fm, pl, line_formats{fm_ind}, point_formats{fm_ind}, 0.05);
        end
        set(ph, 'XLimMode', 'manual');
        set(ph, 'YScale', 'log');
        ylim = get(ph, 'YLim');
        set(ph, 'YLim', [ylim(2)/100, ylim(2)*1.5]);
        ylim = get(ph, 'YLim');
        p = patch([exact_starts(pl) exact_ends(pl) exact_ends(pl) exact_starts(pl)], [10^-5 10^-5 10^5 10^5], 'y', 'FaceColor', [0.7 0.7 0.7], 'FaceAlpha', 0.25, 'EdgeColor', 'none');
        uistack(p, 'bottom');
        sgtitle('Hard');
    end
    ymax = 10^2;
    ymin = 10^-1;
    for pl = 1:Ne
        ph = subplot(8+1, ceil(Ne/8), pl); hold on;
        ph.YLim = [ymin, ymax];
        xs = (exact_starts(pl) + exact_ends(pl)) / 2 + [-0.01, 0, 0.01];
        ys = [ymin, ymin*10^0.5, ymin];
        myplot(xs, ys, '-', 'Color', [0.3 0.3 0.3], 'LineWidth', 1.5);
        for fmi = 1:length(fms(1:3))
            xs = mids(pl, fmi) + [-0.01, 0, 0.01];
            ys = [ymin*10^0.5, ymin, ymin*10^0.5];
            myplot(xs, ys, point_formats{fmi}, 'LineWidth', 1.5);
        end

        edge_pts = mesh.bedgestream(mesh.bedgestream(:, 3) == pl, 4);
        xts = (edge_pts - edge_pts(1) );
        xticks(xts ./ xts(end));
        tickstrs = strsplit(sprintf("%.1f,", 1000*xts), ",");
        tickstrs = tickstrs(1:end-1);
        mask = zeros(size(tickstrs));
        mask([1, 1+ ceil(length(tickstrs(2:end-1)) / 3), end - ceil(length(tickstrs(2:end-1)) / 3), end]) = 1;
        tickstrs(not(mask)) = "";
        new_ax = gca;
        new_ax.XTickLabels = tickstrs;
    end
    subplot(8+1, ceil(Ne/8), (8+1)*ceil(Ne/8)-0.5);
    mylegend(line_formats(1:3), {'CEM', 'PL', 'PH'}, 'LineWidth', 1.5);
    legend('Location', 'northwest', 'Orientation', 'Horizontal'); axis off;

    Contact_resistors = 1./[ints]
    log_contact_conductances_mean_hard(:, end+1) = mean(log(1./Contact_resistors))'
    log_contact_conductances_sd_hard(:, end+1) = sqrt(var(log(1./Contact_resistors)))'
    midpoint_error_squares = vecnorm((exact_centers - mids) .* get_el_lens(mesh))
    midpoint_errors_hard(:, end+1) = mean(abs(exact_centers - mids).*get_el_lens(mesh))'

    all_fms_hard{el_extension_mm+1} = fms;

end

function draw_cond(fm)
    fm.draw_conductivity_field;
%     label_es(fm.mesh, log10(max(fm.domain_parametrization.get_params_as_conductivity())).*ones(size(fm.mesh.p, 2), 1), 1);
    colorbar off;
    colormap(mycolormap);
    symmetriccolorbar(-3.0);
    caxis([-3.8, -2.2]);
end
