finalerrs = [];
P1CEM_constant_mids = maybeor('P1CEM_constant_mids', []);
P1CEM_constant_finalerrs = maybeor('P1CEM_constant_finalerrs', []);

for prior_ls = 0.003;
for prior_ws = 500;

el_parametrization = P1CEM_square_conductivity_electrode_parametrization(mesh);
el_parametrization.set_resistors(1000);
el_parametrization.boundary_conductance_limits = [0, 10^5];

electrodes = Functional_electrodes(mesh, el_parametrization);

dom_parametrization = Constant_log_domain_parametrization(mesh);
dom_parametrization.conductivity_bounds = [10^-6 10^6];
dom_parametrization.set_constant(10^-3)

fm = CEM_forward_model(mesh, dom_parametrization, electrodes, current_plan);


fm.use_log = 1;
for snip = 1 % Figures
    if maybeor('not(DISABLE_DRAW)', 0)
        %h_cond = figure(1); colormap(mycolormap(100)); clf;
        h_iter = figure(2); colormap('gray'); clf;
        %h_grad = figure(3); colormap('gray'); clf;
        h_step = figure(4); colormap(mycolormap(255)); clf;
%         h_exact = figure(5); colormap('gray'); clf;
        h_electrodes = figure(6); clf;
        %h_volts = figure(7); clf;
    end
end

for snip = 1 % Plotting
    if maybeor('not(DISABLE_DRAW)', 0)
        set(0, 'CurrentFigure', h_iter); clf;
        fm.draw_conductivity_field(); colorbar; view(0, 90); labels; title('Iter conductivity'); drawnow;
    end
end
electrodes.label_electrodes(1, mesh, [zeros(fm.solution_structure.domain(1) + electrodes.Ne, 1)], fm.get_scaler());

x0 = [fm.domain_parametrization.get_params();
      zeros(el_parametrization.Nperturb(), 1);];

prior_l = prior_ls;
prior_w = prior_ws;
electrode_prior = gauss_P1CEM_dist_prior(mesh, el_parametrization, prior_l, prior_w);
electrode_prior_inv = inv(electrode_prior + 10^-5 * eye(electrodes.Nperturb));

tikhonov = [eye(fm.perturbation_structure.domain, fm.perturbation_structure.domain), zeros(fm.perturbation_structure.domain, fm.perturbation_structure.electrode_conductances)
            zeros(fm.perturbation_structure.electrode_conductances, fm.perturbation_structure.domain), electrode_prior_inv];

try chol(tikhonov);
catch ME
    disp(sprintf("prior l = %f, w = %f resulted in non-posdef tikhonov.\n", prior_l, prior_w));
    continue
end

L = chol(tikhonov);

alpha_domain = 0;
alpha_electrode = 1;
a_vec = [alpha_domain * ones(fm.perturbation_structure.domain, 1);
         alpha_electrode * ones(fm.perturbation_structure.electrode_conductances, 1)];
aLL = sqrt(a_vec .* a_vec') .* (L' * L);


EIT_minimization();


finalerrs(end+1, :) = [mean(fm.domain_parametrization.get_params_as_conductivity()) / 0.05, ...
                       norm(accuratevolts(:) - volts(:)), ...
                       (fm.domain_parametrization.get_params() - x0(1:fm.perturbation_structure.domain))' * aLL(1:fm.perturbation_structure.domain, 1:fm.perturbation_structure.domain) * (fm.domain_parametrization.get_params() - x0(1:fm.perturbation_structure.domain)), ...
                       (fm.electrode_model.parametrization.get_params() - x0(fm.perturbation_structure.domain+1:end))' * aLL(fm.perturbation_structure.domain+1:end, fm.perturbation_structure.domain+1:end) * (fm.electrode_model.parametrization.get_params() - x0(fm.perturbation_structure.domain+1:end)), ...
                       prior_l, prior_w]

[~, temp_mids] = fm.electrode_model.conductance_integrals();
P1CEM_constant_mids(:, end+1) = temp_mids;
P1CEM_constant_finalerrs(end+1, :) = finalerrs(end, :);
end % prior_ws
end % prior_ls

%
% %%%%%%%%%%% % Helpers
%
function volts = evaluate(fme)
    fme.assemble_system();
    fme.solve_potentials();
    volts = fme.get_NtoD();
end
