finalerrs = [];
Peak_constant_mids = maybeor('Peak_constant_mids', []);
Peak_constant_finalerrs = maybeor('Peak_constant_finalerrs', []);

el_parametrization = Conductance_peak_electrode_parametrization(mesh);
el_parametrization.set_resistors(1000);
el_parametrization.limits.h = [0, 10^5];

electrodes = Functional_electrodes(mesh, el_parametrization);

dom_parametrization = Constant_log_domain_parametrization(mesh);
dom_parametrization.conductivity_bounds = [10^-6 10^6];
dom_parametrization.set_constant(10^-3);

fm = CEM_forward_model(mesh, dom_parametrization, electrodes, current_plan);


fm.use_log = 1;
for snip = 1 % Figures
    if maybeor('not(DISABLE_DRAW)', 0)
        %h_cond = figure(1); colormap(mycolormap(100)); clf;
        h_iter = figure(2); colormap('gray'); clf;
        %h_grad = figure(3); colormap('gray'); clf;
        h_step = figure(4); colormap(mycolormap(255)); clf;
%         h_exact = figure(5); colormap('gray'); clf;
%         h_electrodes = figure(6); clf;
        %h_volts = figure(7); clf;
    end
end

el_perturb_width = coly(maybeor('el_width ./ (el_extension_width + el_width)', 0.5));

x0 = [fm.domain_parametrization.get_params();
      zeros(size(electrodes.parametrization.params.h));
      0.5 * ones(size(electrodes.parametrization.params.l, 1), 1); % == Center
      el_perturb_width .* ones(size(electrodes.parametrization.params.w, 1), 1)];

% These result in less residual than CEM with no regulation.
% prior_h = 10^-12.0;
% prior_l = 10^-4.0;
% prior_w = 10^-5.0;
prior_h = 10^-6.0;
prior_l = 10^-3.0;
prior_w = 10^-4.0;


Ne = fm.electrode_model.Ne;
electrode_prior = [prior_h * eye(Ne), zeros(Ne), zeros(Ne);
                   zeros(Ne), prior_l * eye(Ne), zeros(Ne);
                   zeros(Ne), zeros(Ne), prior_w * eye(Ne);];
L_electrode = chol(electrode_prior);

tikhonov = [eye(fm.perturbation_structure.domain, fm.perturbation_structure.domain), zeros(fm.perturbation_structure.domain, fm.perturbation_structure.electrode_conductances)
            zeros(fm.perturbation_structure.electrode_conductances, fm.perturbation_structure.domain), electrode_prior];

L = chol(tikhonov);

alpha_domain = 0;
alpha_electrode = 1;
a_vec = [alpha_domain * ones(fm.perturbation_structure.domain, 1);
         alpha_electrode * ones(fm.perturbation_structure.electrode_conductances, 1)];
aLL = sqrt(a_vec .* a_vec') .* (L' * L);


EIT_minimization();

finalerrs(end+1, :) = [mean(fm.domain_parametrization.get_params_as_conductivity()) / 0.05, ...
                       norm(accuratevolts(:) - volts(:)), ...
                       (fm.domain_parametrization.get_params() - x0(1:fm.perturbation_structure.domain))' * aLL(1:fm.perturbation_structure.domain, 1:fm.perturbation_structure.domain) * (fm.domain_parametrization.get_params() - x0(1:fm.perturbation_structure.domain)), ...
                       (fm.electrode_model.parametrization.get_params() - x0(fm.perturbation_structure.domain+1:end))' * aLL(fm.perturbation_structure.domain+1:end, fm.perturbation_structure.domain+1:end) * (fm.electrode_model.parametrization.get_params() - x0(fm.perturbation_structure.domain+1:end)), ...
                       prior_h, prior_l, prior_w]


[~, temp_mids] = fm.electrode_model.conductance_integrals();
Peak_constant_mids(:, end+1) = temp_mids;
Peak_constant_finalerrs(end+1, :) = finalerrs(end, :);
