final_h = myfigure(60); clf;

extensions = [0 10 20];
subplot(3, 4, 1);

old_titles = {'CEM', 'P1CEM', 'Peak'};
new_titles = {'CEM', 'PL', 'PH'};
extension_names = {'0', '12', '22'};

min_y = inf;
max_y = -inf;
max_x = -inf;

final_h.Position = [0 0 1100 600];

for extension_id = 1:length(extensions)
    extension = extensions(extension_id);
    
    for fm_id = 1:3
        fms = all_fms_hard{extension+1};
        fm = fms(fm_id);
        
        sp_h = subplot(length(extensions), 3 + 1, (extension_id - 1) * 4 + fm_id, 'Parent', final_h);
        ext_f_h = myfigure(100 + extension);
        fm_h_id = find_child_index(ext_f_h.Children, old_titles{fm_id});
        new_ax = copyobj(ext_f_h.Children(fm_h_id), final_h);
        new_pos = get(new_ax, 'Position') .* [0 0 0 0] + get(sp_h, 'Position') .* [1 1 1 1] + 0.08*[-0.5 -0.5 1 1*6/11];
        set(new_ax, 'Position', new_pos);
        delete(sp_h);
        new_ax.Title.String = sprintf("");
        if extension_id == 1
            new_ax.Title.String = [new_titles{fm_id}];
        end
        if fm_id == 1
            new_ax.YLabel.String = {sprintf("%s mm", extension_names{extension_id})};
            new_ax.YLabel.FontWeight = 'bold';
        end
        set(0, 'CurrentFigure', final_h);
        set(final_h, 'CurrentAxes', new_ax);

        axis equal;
        axis([-0.20 0.20 -0.15 0.15]);
        el_z = max(fm.nodewise_conductivity());
        label_es(fm.mesh, max(el_z, fm.nodewise_conductivity()), 1, fm.get_scaler(), 'Disable_numbers', 1);

        min_y = min(new_ax.Position(2), min_y);
        max_y = max(new_ax.Position(2) + new_ax.Position(4), max_y);
        max_x = max(new_ax.Position(1) + new_ax.Position(3), max_x);

        xticklabels("")
        yticklabels("")

        p_h = findobj(new_ax.Children, 'Type', 'Patch');
        p_h.EdgeAlpha = 0;
    end
end

sp_h = subplot(length(extensions), 3 + 1, 8, 'Parent', final_h);
colorbar;
sp_h.Visible = 'off';
cbar = findobj(final_h.Children, 'Type', 'ColorBar');
caxis([-3.8, -2.2]);
cbar.Ticks = [-3.8 -3.4 -3 -2.6 -2.2];
cbar.TickLabels = {'10^{-2.5}', '10^{-2.1}', '10^{-1.7}', '10^{-1.3}', '10^{-0.9}'}; % Division by 0.05 (=tank height) amounts to shift by 1.299 in the exponent.
cbar.Position(2) = min_y;
cbar.Position(4) = max_y - min_y;
cbar.Position(1) = max_x + 0.02;
colormap(mycolormap(255));

savefigure('reconstructions', '-r600');


function ind = find_child_index(children, title)
    for ind = 1:length(children)
        child = children(ind);
        if isa(child, 'matlab.graphics.axis.Axes')
            if strcmp(child.Title.String, title)
                return
            end
        end
    end
end
