incfh = myfigure(46); % Domain with inclusions
empfh = myfigure(47); % Empty domain

fh50 = myfigure(50); clf;

wanted_titles = strsplit(sprintf('Electrode %d\n', [1 3 5 7 9 11 13 15]), '\n');
wanted_titles = wanted_titles(1:end-1);
subplot(length(wanted_titles) + 1, 2, 1);

ymax = -inf;
ymin = inf;

ind = 1;
for tit = wanted_titles
    elid = str2double(extractBetween(tit{1}, regexp(tit{1}, "[0-9]*"), strlength(tit{1})));
    
    sph = subplot(length(wanted_titles) + 1, 2, ind, 'parent', fh50);
    empfhind = find_child_index(empfh.Children, tit{1});
    new_ax = copyobj(empfh.Children(empfhind), fh50);
    new_pos = get(new_ax, 'Position') .* [0 0 0 0] + get(sph, 'Position') .* [1 1 1 1];
    set(new_ax, 'Position', new_pos);%get(sph, 'Position'));
    delete(sph);
    new_ax.Title.String = sprintf("");
    if ind == 1
        new_ax.Title.String = ["Homogeneous domain", " "];
    end
    new_ax.YLabel.String = {tit{1}, " "};
    new_ax.YLabel.FontWeight = 'bold';

    edge_pts = mesh.bedgestream(mesh.bedgestream(:, 3) == elid, 4);
    xts = (edge_pts - edge_pts(1) );
    new_ax.XTick = xts ./ xts(end);

    tickstrs = strsplit(sprintf("%.1f,", 1000*xts), ",");
    tickstrs = tickstrs(1:end-1);
    mask = zeros(size(tickstrs));
    mask([1, 1+ ceil(length(tickstrs(2:end-1)) / 3), end - ceil(length(tickstrs(2:end-1)) / 3), end]) = 1;
    tickstrs(not(mask)) = "";
    new_ax.XTickLabels = tickstrs;

    ymax = max(ymax, new_ax.YLim(2));
    ymin = min(ymin, new_ax.YLim(1));
    
    new_ax.YTick = [0.1 1 10 100];
    new_ax.YTickLabels = [{"10^{-1}"}, {""}, {"10^{1}"}, {""}];
    new_ax.YGrid = 1;
    new_ax.YMinorGrid = 0;
    
    ind = ind + 2;
end

ind = 2;
for tit = wanted_titles
    elid = str2double(extractBetween(tit{1}, regexp(tit{1}, "[0-9]*"), strlength(tit{1})));

    sph = subplot(length(wanted_titles) + 1, 2, ind, 'parent', fh50);
    incfhind = find_child_index(incfh.Children, tit{1});
    new_ax = copyobj(incfh.Children(incfhind), fh50);
    new_pos = get(new_ax, 'Position') .* [0 0 0 0] + get(sph, 'Position') .* [1 1 1 1];
    set(new_ax, 'Position', new_pos);%get(sph, 'Position'));
    new_ax.Title.String = sprintf("");
    delete(sph)
    if ind == 2
        new_ax.Title.String = ["Domain with inclusions", " "];
    end
    edge_pts = mesh.bedgestream(mesh.bedgestream(:, 3) == elid, 4);
    xts = (edge_pts - edge_pts(1) );
    new_ax.XTick = xts ./ xts(end);

    tickstrs = strsplit(sprintf("%.1f,", 1000*xts), ",");
    tickstrs = tickstrs(1:end-1);
    mask = zeros(size(tickstrs));
    mask([1, 1+ ceil(length(tickstrs(2:end-1)) / 3), end - ceil(length(tickstrs(2:end-1)) / 3), end]) = 1;
    tickstrs(not(mask)) = "";
    new_ax.XTickLabels = tickstrs;
    
    ymax = max(ymax, new_ax.YLim(2));
    ymin = min(ymin, new_ax.YLim(1));
    
    new_ax.YTick = [0.1 1 10 100];
    new_ax.YTickLabels = [{"10^{-1}"}, {""}, {"10^{1}"}, {""}];
    new_ax.YGrid = 1;
    new_ax.YMinorGrid = 0;
    
    ind = ind + 2;
end

for child_ind = 1:length(fh50.Children)
    if isa(fh50.Children(child_ind), 'matlab.graphics.axis.Axes')
        fh50.Children(child_ind).YLim = [ymin, ymax];
    end
end

subplot(length(wanted_titles) + 1, 2, length(wanted_titles)*2 + 1.5);
mylegend(line_formats(1:3), {'CEM', 'PL', 'PH'}, 'LineWidth', 1.5);
legend('Location', 'north', 'Orientation', 'Horizontal'); axis off;
fh50.Position = [6 6 950 1109]

savefigure('contactcomparison')

function ind = find_child_index(children, title)
    for ind = 1:length(children)
        child = children(ind);
        if isa(child, 'matlab.graphics.axis.Axes')
            if strcmp(child.Title.String, title)
                return
            end
        end
    end
end