function label_es(mesh, solution, solution_id, scaler, varargin)
    if nargin == 0
        mesh = evalin('base', 'mesh');
        if evalin('base', 'exist("fm")')
            solution = evalin('base', 'fm.get_domain_potentials()');
        else
            solution = zeros(size(mesh.p, 2), 1);
        end

        solution_id = 1;
    end
    if nargin < 4
        scaler = @(x) x;
    end
    
    parser = inputParser;
    addOptional(parser, 'Disable_numbers', 0);
    parse(parser, varargin{:});
    
    hold on;

    [electrodes, IAbegin] = unique(mesh.bedgestream(:, 3), 'first');
    [~, IAend] = unique(mesh.bedgestream(:, 3), 'last');
    IAbegin = IAbegin(electrodes ~= 0);
    IAend = IAend(electrodes ~= 0);
    electrodes = electrodes(electrodes ~= 0);
    midpoints = (mesh.bedgestream(IAend, 4) - mesh.bedgestream(IAbegin, 4))./2 + mesh.bedgestream(IAbegin, 4);
    vq = interp1(mesh.bedgestream(:, 4), [mesh.p(:, mesh.bedgestream(:, 1))', scaler(solution(mesh.bedgestream(:, 1), solution_id))], midpoints);

    delete_old_e_marks();
    
    bbox = [min(mesh.p(1, :)), max(mesh.p(1, :)); min(mesh.p(2, :)), max(mesh.p(2, :))];
    f_size = 15;
    ax = gca();
    old_axis = axis;
    old_units = ax.Units;
    ax.Units = 'pixels';
    axis tight;
    pixel2unitx = (bbox(1,2) - bbox(1,1)) / ax.Position(3);
    pixel2unity = (bbox(2,2) - bbox(2,1)) / ax.Position(4);

    zshift = scaler((max(solution(:, solution_id))) - scaler(min(solution(:, solution_id)))) / 10;
    for el_id_id = 1:max(electrodes)
        el_id = electrodes(el_id_id);
        el_ps = mesh.bedgestream(mesh.bedgestream(:, 3) == el_id, 1);

        mid_p1 = el_ps(max(1, floor(end/2)));
        mid_p2 = el_ps(min(end, ceil(end/2 + 0.5)));
        [~, p1_ts]  = find(mesh.t == mid_p1);
        [~, p2_subind] = find(mesh.t(:, p1_ts) == mid_p2);
        tind = p1_ts(p2_subind);
        in_p_ind = mesh.t(mesh.t(:, tind) ~= mid_p1 & mesh.t(:, tind) ~= mid_p2, tind);
        v_in = mesh.p(:, mid_p1) - mesh.p(:, in_p_ind);
        v_edge = mesh.p(:, mid_p2) - mesh.p(:, mid_p1);
        normal = v_in - (v_in' * v_edge) / norm(v_edge) * v_edge;
        normal = normal./norm(normal);

        line_col = [0.60, 0.60, 0.60];
        if (el_id == 1)
            line_col = [0.20, 0.20, 0.20];
        end
        plot3(mesh.p(1, el_ps), mesh.p(2, el_ps), scaler(solution(el_ps, solution_id)), 'color', line_col, 'linewidth', 2);
        n_letters = strlength(sprintf('%d', el_id));
        x_shift = (-n_letters * 2/5 * f_size * pixel2unitx) + n_letters * f_size * 4/5 * pixel2unitx * normal(1);
        y_shift = 1.5 * f_size * pixel2unity * normal(2);
        if not(parser.Results.Disable_numbers)
            text(vq(el_id, 1) + x_shift, vq(el_id, 2) + y_shift, vq(el_id, 3) + zshift, sprintf("%d", el_id), 'color', [0.5, 0.5, 0.5], 'FontSize', f_size);
        end
    end
    axis(old_axis)
    ax.Units = old_units;
end

function delete_old_e_marks()
    ax = gca();
    texts = findobj(ax.Children, 'type', 'text');
    to_del = [];
    for (t_ind = 1:length(texts))
        if regexp(texts(t_ind).String, "[0-9]+")
            to_del(end+1) = t_ind;
        end
    end
    delete(texts(to_del));
end
