function els = extend_electrodes(handles, els, extend_elements, skew)
    if extend_elements == 0 % Extend electrodes by at least given amount
        return
    end
    
    if length(extend_elements) == 1
        extend_elements = extend_elements .* ones(max(els), 1);
    end
    if length(skew) == 1
        skew = skew .* ones(max(els), 1);
    end

    data = [els', handles'];

    iter = Bidir_circ_iter(data);
    % Extend forward
    fwd_dists = extend_elements .* skew;
    bwd_dists = extend_elements .* (1 - skew);
    
    iter = directional_extend(iter, @(it) it.next(), @(it) it.get_prev(), @(it, val) it.set_prev(val), fwd_dists);
    iter = directional_extend(iter, @(it) it.prev(), @(it) it.get_next(), @(it, val) it.set_next(val), bwd_dists);
    els = iter.data(:, 1)';    
end

function iter = directional_extend(iter, next_func, get_prev_func, set_prev_func, dists)
    el_pen = 0;
    step_count = 0;

    while sum(dists > 0) > 0
        prev_val = get_prev_func(iter);
        iter_val = iter.get();

        if (el_pen == 0 && iter_val(1) == 0)
            % Nothing to draw and not on electrode. Do nothing.
        elseif (el_pen == 0 && iter_val(1) ~= 0)
            % Reached next electrode. Change pen color to match this electrode.
            el_pen = iter_val(1);
        elseif (el_pen ~=0 && iter_val(1) == el_pen)
            % Not at the end of electrode. Do nothing.
        elseif (el_pen ~= 0 && iter_val(1) == 0)
            % Extend electrode forward. Change pen color to 0 if sufficiently extended.
            iter_val(1) = el_pen;
            iter.set(iter_val);
            len = norm(prev_val(2:3) - iter_val(2:3));
            dists(el_pen) = dists(el_pen) - len;
            if (dists(el_pen) <= 0)
                el_pen = 0;
            end
        elseif (el_pen ~= 0 && iter_val(1) ~= 0 && iter_val(1) ~= el_pen)
            % Trying to draw on same electrode. Don't extend, set previous
            % to zero to ensure a gap between electrodes and warn.
            prev_val(1) = 0;
            set_prev_func(iter, prev_val);
            warning("Electrode extension about to overlap electrodes %d and %d. Extension not completed for %d.", el_pen, iter_val(1), el_pen);
            dists(el_pen) = 0;
            el_pen = 0;
        end
        iter = next_func(iter);
        
        % Avoid infinite loop:
        step_count = step_count + 1;
        if (step_count > 2 * iter.N)
            error("Can't extend electrodes as much as requested.")
        end
    end
end
