function b_params = get_boundary_parametrization(mesh, Xb)
    % Arc length calculations
    Nibp = size(Xb, 2);

    edge_end_dists = mesh.bedgestream(:, 5);
    edge_start_dists = mesh.bedgestream(:, 4);

    b_params.boundary_l = sqrt((mesh.affine.Abx * Xb).^2 + (mesh.affine.Aby * Xb).^2) + edge_start_dists;

    [electrodes, IAbegin] = unique(mesh.bedgestream(:, 3), 'first');
    [~, IAend] = unique(mesh.bedgestream(:, 3), 'last');
    IAbegin = IAbegin(electrodes ~= 0);
    IAend = IAend(electrodes ~= 0);
    IAend = IAend - 1;

    electrodewise_begin_distances = edge_start_dists(IAbegin);
    electrodewise_end_distances = edge_end_dists(IAend);
    b_params.elmask = (mesh.bedgestream(:, 3) ~= 0);
    b_params.elmask = min(b_params.elmask, circshift(b_params.elmask, [-1 0])); % Remove last node as it correspond to the following edge.
    b_params.electrode_l = zeros(size(b_params.boundary_l));
    b_params.electrode_l(b_params.elmask, :) = (b_params.boundary_l(b_params.elmask, :) - electrodewise_begin_distances(mesh.bedgestream(b_params.elmask, 3))) ./ (electrodewise_end_distances(mesh.bedgestream(b_params.elmask, 3)) - electrodewise_begin_distances(mesh.bedgestream(b_params.elmask, 3)));
    b_params.gXb{1} = bsxfun(@plus, mesh.affine.Abx*Xb, mesh.affine.bbx);
    b_params.gXb{2} = bsxfun(@plus, mesh.affine.Aby*Xb, mesh.affine.bby);
    b_params.Xb = Xb;
    b_params.electrode_inds = repmat(mesh.bedgestream(:, 3), 1, Nibp) .* b_params.elmask; % Per quadrature point, but any column is equivalent to per edge.
end
