function [mesh] = get_torso_mesh(extend_elements, skew)
    if nargin < 2
        skew = 0.5;
    end
    if nargin < 1
        extend_elements = 0;
    end

    [handles, els] = load_datas();
    handles = handles ./ 100; % cm to m

    els = extend_electrodes(handles, els, extend_elements, skew);

    handles = handles(:, 1:end-1);
    els = els(:, 1:end-1);
    
    triangle_options = TriangleOpts();
    triangle_options.area = 0.00005;
    
    if (programexists("triangle"))
        old_wd = pwd;
        tempdir = tempname(pwd);
        mkdir(tempdir);
        cd(tempdir);
        saveBoundary("boundary", handles, els);
        trianglecmd = strcat('triangle', triangle_options.getArguments("boundary.poly"));
        if isunix
            [ret, out] = system(trianglecmd);
        elseif ispc
            [ret, out] = system(sprintf("bash -c ""%s""", trianglecmd));
        end
        opts = delimitedTextImportOptions('NumVariables', 4, 'LeadingDelimitersRule', 'ignore', 'Delimiter', ' ', 'DataLines', 2, 'CommentStyle', '#', 'ConsecutiveDelimitersRule', 'join', 'VariableTypes', {'double', 'double', 'double', 'double'});
        t = readmatrix('boundary.1.ele', opts);
        t = t(:, 2:end)';
        opts = delimitedTextImportOptions('NumVariables', 5, 'LeadingDelimitersRule', 'ignore', 'Delimiter', ' ', 'DataLines', 2, 'CommentStyle', '#', 'ConsecutiveDelimitersRule', 'join', 'VariableTypes', {'double', 'double', 'double', 'double', 'double'});
        pdata = readmatrix('boundary.1.node', opts);
        p = pdata(:, 2:3)';
        boundary_indices = pdata(pdata(:, 5) == 1, 4);
        boundary_indices = [pdata(pdata(:, 5) == 1, 1), boundary_indices];
        mesh = inittri(p, t, boundary_indices);
        cd(old_wd);
        delete(strcat(tempdir, "/boundary.poly"));
        delete(strcat(tempdir, "/boundary.1.node"));
        delete(strcat(tempdir, "/boundary.1.ele"));
        fclose('all');
        %delete(strcat(tempdir, "/boundary.1.edge"));
        if isunix
            rmdir(tempdir);
        elseif ispc
            system(sprintf('rmdir %s', tempdir));
        end
    end

    Ne = max(mesh.bedgestream(:, 3));
end


function saveBoundary(filename, handles, els)
    fhandle = fopen(strcat(filename, ".poly"), 'w');
    fprintf(fhandle, '#Vertices\n%d 2 1 0\n', size(handles, 2));
    for row = 1:length(handles)
        fprintf(fhandle, '%d %f %f %d\n', row, handles(1, row), handles(2, row), els(row));
    end
    fprintf(fhandle, '#Segments\n%d\n', size(handles, 2));
    for row = 1:length(handles)-1
        fprintf(fhandle, '%d %d %d\n', row, row, row+1);
    end
    fprintf(fhandle, '%d %d %d\n', length(handles), length(handles), 1);
    fprintf(fhandle, '#holes\n0\n');
    fclose(fhandle);
end


function [handles, electrode_inds] = load_datas()
    load 'mesh_boundary.mat'
    load 'electrodes.mat'

    e_ps = [x_elek y_elek];
    e_ps = e_ps(1:end-1, :);
    ps = [x; y]';
    e_row_i = 1;
    el_list = zeros(size(ps, 1), 1);
    next_el_ind = 0;

    for p_i = 1:size(ps, 1)
        el_list(p_i) = next_el_ind;
        if norm(ps(p_i, :) - e_ps(e_row_i, :)) < 10^-4
            if (next_el_ind == 0)
                next_el_ind = mod(e_row_i, 2) * (1+e_row_i)/2;
                el_list(p_i) = next_el_ind;
            else
                el_list(p_i) = next_el_ind;
                next_el_ind = mod(e_row_i, 2) * (1+e_row_i)/2;
            end
            e_row_i = e_row_i + 1;
            if e_row_i > size(e_ps, 1)
                handles = ps';
                electrode_inds = el_list';
                return;
            end
        end
    end

    % Remove repeated last element
    handles = handles(:, 1:end-1);
    electrode_inds = electrode_inds(:, 1:end-1);
end